package cn.estsh.i3plus.ext.mes.api.base;

import cn.estsh.i3plus.ext.mes.pojo.bean.MesShiftProd;
import cn.estsh.i3plus.ext.mes.pojo.model.ShiftProdModel;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import io.swagger.annotations.ApiOperation;

import java.util.List;

/**
 * @Author: wangjie
 * @CreateDate: 2021/01/19 15:06 下午
 * @Description:
 **/
public interface IShiftProdService {

    /**
     * 按条件分页查询班次作业项目信息
     *
     * @param shiftProd 查询条件
     * @param pager 分页条件
     * @return 班次作业项目分页数据
     */
    @ApiOperation(value = "按条件分页查询班次作业项目信息", notes = "按条件分页查询班次作业项目信息")
    ListPager<ShiftProdModel> queryShiftProdByPager(ShiftProdModel shiftProd, Pager pager);

    /**
     * 新增班次作业项目信息
     * @param shiftProd 班次作业项目信息
     * @return 班次作业项目数据
     */
    @ApiOperation(value = "新增班次作业项目信息", notes = "新增班次作业项目信息")
    MesShiftProd insertShiftProd(MesShiftProd shiftProd);

    /**
     * 更新班次作业项目信息
     * @param shiftProd 班次作业项目信息
     * @return 班次作业项目数据
     */
    @ApiOperation(value = "更新班次作业项目信息", notes = "更新班次作业项目信息")
    MesShiftProd updateShiftProd(MesShiftProd shiftProd);

    /**
     * 根据id启用或禁用班次作业项目信息
     * @param id 班次作业项目信息ID
     * @param status 状态
     * @param organizeCode 组织代码
     * @param userName 操作人
     * @return 结果
     */
    @ApiOperation(value = "根据id启用或禁用班次作业项目信息", notes = "根据id启用或禁用班次作业项目信息")
    int updateShiftProdValidById(Long id, Integer status, String organizeCode, String userName);

    /**
     * 根据id删除班次作业项目信息
     * @param id 班次作业项目信息ID
     * @param organizeCode 组织代码
     * @param userName 操作人
     * @return 结果
     */
    @ApiOperation(value = "根据id删除班次作业项目信息", notes = "根据id删除班次作业项目信息")
    int deleteShiftProdById(Long id, String organizeCode, String userName);

    /**
     * 获取班次作业项目信息
     * @param organizeCode 组织代码
     * @return 班次作业项目信息
     */
    @ApiOperation(value = "获取班次作业项目信息", notes = "获取班次作业项目信息")
    List<MesShiftProd> getShiftProdList(String organizeCode);

}
