package cn.estsh.i3plus.ext.mes.apiservice.controller.base;

import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.AffiliatedStartConfigService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesAffiliatedStartConfig;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @author yumingxing
 * @version 1.0
 * @date 2021/1/29 14:41
 **/
@RestController
@Api(tags = "工单启动附属启动配置")
@RequestMapping(CommonConstWords.BASE_URL_MES + "/affiliated-start-config")
public class AffiliatedStartConfigController {

    @Autowired
    private AffiliatedStartConfigService mesAffiliatedStartConfigService;
    /**
     * 插入
     *
     * @param mesAffiliatedStartConfig mes自动
     * @return {@link ResultBean}
     */
    @PostMapping(value = "/insert")
    @ApiOperation(value = "新增")
    public ResultBean insert(@RequestBody MesAffiliatedStartConfig mesAffiliatedStartConfig) {
        try {
            //条件验证
            ValidatorBean.beginValid(mesAffiliatedStartConfig)
                    .notNull("affiliatedStartClass", mesAffiliatedStartConfig.getAffiliatedStartClass())
                    .notNull("executeDesc", mesAffiliatedStartConfig.getExecuteDesc())
            ;
            return ResultBean.success("新增成功").setResultObject(mesAffiliatedStartConfigService.insert(mesAffiliatedStartConfig))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 修改
     *
     * @param mesAffiliatedStartConfig mes部分自动降落
     * @return {@link ResultBean}
     */
    @PutMapping(value = "/update")
    @ApiOperation(value = "修改")
    public ResultBean update(@RequestBody MesAffiliatedStartConfig mesAffiliatedStartConfig) {
        try {
            //条件验证
            ValidatorBean.beginValid(mesAffiliatedStartConfig)
                    .notNull("affiliatedStartClass", mesAffiliatedStartConfig.getAffiliatedStartClass())
                    .notNull("executeDesc", mesAffiliatedStartConfig.getExecuteDesc())
            ;
            return ResultBean.success("修改成功").setResultObject(mesAffiliatedStartConfigService.update(mesAffiliatedStartConfig))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


    /**
     * 查询分页
     *
     * @param mesAffiliatedStartConfig mes自动
     * @param pager             寻呼机
     * @return {@link ResultBean}
     */
    @GetMapping(value = "/query")
    @ApiOperation(value = "查询分页")
    public ResultBean queryOrderRule(MesAffiliatedStartConfig mesAffiliatedStartConfig, Pager pager) {
        try {
            return ResultBean.success("查询成功").setListPager(mesAffiliatedStartConfigService.queryByPager(mesAffiliatedStartConfig, pager))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据ID启用或禁用单据规则
     * @param id
     * @param status
     * @return
     */
    @PutMapping(value = "/status-update")
    @ApiOperation(value = "根据ID启用或禁用")
    public ResultBean updateOrderIsValidById(long id, Integer status) {
        try {
            return ResultBean.success("根据ID启用或禁用").setResultObject(mesAffiliatedStartConfigService.updateIsValidById(id,status))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}
