package cn.estsh.i3plus.ext.mes.apiservice.controller.base;

import cn.estsh.i3plus.ext.mes.api.base.INumberRuleExtService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesNumberRuleExt;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.impp.framework.base.controller.MesBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

/**
 * @Author: wangjie
 * @CreateDate: 2021/01/19 15:37
 * @Description:
 **/
@RestController
@RequestMapping(CommonConstWords.BASE_URL_MES)
@Api(tags = "条码生成规则扩展控制层")
public class NumberRuleExtController extends MesBaseController {

    @Autowired
    private INumberRuleExtService numberRuleExtService;

    @GetMapping(value = "/number-rule-ext/query")
    @ApiOperation(value = "按条件分页查询条码生成规则信息")
    public ResultBean queryNumberRuleByPager(MesNumberRuleExt numberRuleExt, Pager pager) {
        try {
            numberRuleExt.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setListPager(numberRuleExtService.queryNumberRuleByPager(numberRuleExt, pager));
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/number-rule-ext/insert")
    @ApiOperation(value = "新增条码生成规则信息")
    public ResultBean insertNumberRule(MesNumberRuleExt numberRuleExt) {
        try {
            ValidatorBean.beginValid(numberRuleExt)
                    .notNull(MesExtConstWords.RULE_CODE, numberRuleExt.getRuleCode())
                    .notNull(MesExtConstWords.NUMBER_RULE, numberRuleExt.getNumberRule())
                    .notNull(MesExtConstWords.SERIALNO_LENGTH, numberRuleExt.getSerialnoLength())
                    .notNull(MesExtConstWords.SERIALNO_INCREMENT, numberRuleExt.getSerialnoIncrement())
                    .notNull(MesExtConstWords.BEGIN_VAL, numberRuleExt.getBeginVal())
                    .notNull(MesExtConstWords.IS_CYCLE, numberRuleExt.getIsCycle());

            numberRuleExt.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            numberRuleExt.setCreateUser(AuthUtil.getSessionUser().getUserName());
            numberRuleExtService.insertNumberRule(numberRuleExt);

            return ResultBean.success("添加成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/number-rule-ext/update")
    @ApiOperation(value = "更新条码生成规则信息")
    public ResultBean updateNumberRule(MesNumberRuleExt numberRuleExt) {
        try {
            ValidatorBean.beginValid(numberRuleExt)
                    .notNull(MesExtConstWords.ID, numberRuleExt.getId())
                    .notNull(MesExtConstWords.RULE_CODE, numberRuleExt.getRuleCode())
                    .notNull(MesExtConstWords.NUMBER_RULE, numberRuleExt.getNumberRule())
                    .notNull(MesExtConstWords.SERIALNO_LENGTH, numberRuleExt.getSerialnoLength())
                    .notNull(MesExtConstWords.SERIALNO_INCREMENT, numberRuleExt.getSerialnoIncrement())
                    .notNull(MesExtConstWords.BEGIN_VAL, numberRuleExt.getBeginVal())
                    .notNull(MesExtConstWords.IS_CYCLE, numberRuleExt.getIsCycle());

            numberRuleExt.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            numberRuleExt.setModifyUser(AuthUtil.getSessionUser().getUserName());
            numberRuleExtService.updateNumberRule(numberRuleExt);
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/number-rule-ext/status-update")
    @ApiOperation(value = "根据id启用或禁用条码生成规则信息")
    public ResultBean updateNumberRuleValidById(Long id, Integer status) {
        try {
            if (StringUtils.isEmpty(id)) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要操作的资源。")
                        .build();
            }
            numberRuleExtService.updateNumberRuleValidById(id, status, AuthUtil.getSessionUser().getUserName());
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/number-rule-ext-all/query")
    @ApiOperation(value = "按条件查询条码生成规则所有有效数据")
    public ResultBean queryNumberRuleAll(MesNumberRuleExt numberRuleExt) {
        try {
            numberRuleExt.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(numberRuleExtService.queryNumberRuleAll(numberRuleExt));
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
