package cn.estsh.i3plus.ext.mes.apiservice.controller.base;

import cn.estsh.i3plus.ext.mes.api.base.IProdPackExtService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesProdPackExt;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.impp.framework.base.controller.MesBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

/**
 * @Author: wangjie
 * @CreateDate: 2021/01/19 15:37
 * @Description:
 **/
@RestController
@RequestMapping(CommonConstWords.BASE_URL_MES)
@Api(tags = "产品包装关系扩展控制层")
public class ProdPackExtController extends MesBaseController {

    @Autowired
    private IProdPackExtService prodPackExtService;

    @GetMapping(value = "/prod-pack-ext/query")
    @ApiOperation(value = "按条件分页查询产品包装关系扩展信息")
    public ResultBean queryProdPackByPager(MesProdPackExt prodPackExt, Pager pager) {
        try {
            prodPackExt.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setListPager(prodPackExtService.queryProdPackByPager(prodPackExt, pager));
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/prod-pack-ext/insert")
    @ApiOperation(value = "新增产品包装关系扩展信息")
    public ResultBean insertProdPack(MesProdPackExt prodPackExt) {
        try {
            prodPackExt.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            prodPackExt.setCreateUser(AuthUtil.getSessionUser().getUserName());
            prodPackExtService.insertProdPack(prodPackExt);

            return ResultBean.success("添加成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/prod-pack-ext/update")
    @ApiOperation(value = "更新产品包装关系扩展信息")
    public ResultBean updateProdPack(MesProdPackExt prodPackExt) {
        try {
            ValidatorBean.beginValid(prodPackExt)
                    .notNull(MesExtConstWords.ID, prodPackExt.getId());

            prodPackExt.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            prodPackExt.setModifyUser(AuthUtil.getSessionUser().getUserName());
            prodPackExtService.updateProdPack(prodPackExt);
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/prod-pack-ext/status-update")
    @ApiOperation(value = "根据id启用或禁用产品包装关系扩展信息")
    public ResultBean updateProdPackValidById(Long id, Integer status) {
        try {
            if (StringUtils.isEmpty(id)) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要操作的资源。")
                        .build();
            }
            prodPackExtService.updateProdPackValidById(id, status, AuthUtil.getSessionUser().getUserName());
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
