package cn.estsh.i3plus.ext.mes.apiservice.controller.base.jx;

import cn.estsh.i3plus.ext.mes.api.base.jx.IManageCodeService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesManageCode;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @Author: xinwang.yi
 * @CreateDate: 2021/01/18 13:33 下午
 * @Description:
 **/
@RestController
@Api(tags = "管理码 控制层")
@RequestMapping(CommonConstWords.BASE_URL_MES + "/manage-code")
public class ManageCodeController {
    public static final Logger LOGGER = LoggerFactory.getLogger(ManageCodeController.class);

    @Autowired
    private IManageCodeService manageCodeService;

    @PostMapping(value = "/insert")
    @ApiOperation(value = "添加MES_管理码信息")
    public ResultBean insertMesManageCode(MesManageCode mesManageCode) {
        try {
            //条件验证
            ValidatorBean.beginValid(mesManageCode)
                    .notNull(MesExtConstWords.MANAGE_CODE,mesManageCode.getManageCode())
                    .notNull(MesExtConstWords.WORK_CENTER_CODE,mesManageCode.getWorkCenterCode())
                    .notNull(MesExtConstWords.WORK_CENTER_NAME, mesManageCode.getWorkCenterName());
            mesManageCode.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            manageCodeService.insertMesManageCode(mesManageCode);
            return ResultBean.success("添加成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException imppException) {
            LOGGER.error(imppException.getErrorMsg() + "：{}", imppException.getErrorDetail(), imppException);
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/update")
    @ApiOperation(value = "更新MES_管理码项目")
    public ResultBean updateMesManageCode(MesManageCode mesManageCode) {
        try {
            //条件验证
            ValidatorBean.beginValid(mesManageCode)
                    .notNull(MesExtConstWords.MANAGE_CODE,mesManageCode.getManageCode())
                    .notNull(MesExtConstWords.WORK_CENTER_CODE,mesManageCode.getWorkCenterCode())
                    .notNull(MesExtConstWords.WORK_CENTER_NAME, mesManageCode.getWorkCenterName());

            if (mesManageCode.getId() == null || mesManageCode.getId().longValue() < 1) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要操作的资源。")
                        .build();
            }

            manageCodeService.updateMesManageCode(mesManageCode, AuthUtil.getSessionUser().getUserName());

            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/query")
    @ApiOperation(value = "按条件分页查询MES_管理码项目信息", notes = "按条件分页查询MES_管理码项目信息")
    public ResultBean queryMesManageCodeByPager(MesManageCode mesManageCode, Pager pager) {
        try {
            mesManageCode.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            return ResultBean.success().setListPager(manageCodeService.queryMesManageCodeByPager(mesManageCode, pager))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/update-status-batch")
    @ApiOperation(value = "批量禁用/启用详情", notes = "批量禁用/启用详情")
    public ResultBean updateMesManageCodeStatus(String[] idsStr, int status) {
        try {
            ConvertBean.modelSafeArrayNumber(idsStr, true);
            //登陆用户
            SessionUser user = AuthUtil.getSessionUser();

            manageCodeService.updateMesManageCodeStatus(idsStr, status, user.getUserName());
            return ResultBean.success("修改成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}
